open Syntax
open Log

type atom = int
type literal = int

let next_atom = ref 1
let atom_hash : (fhashval,int) Hashtbl.t = Hashtbl.create 1000
let atom_hash_rev : (int,ftrma) Hashtbl.t = Hashtbl.create 1000

let new_atom_actions = ref []

let max_atom () = (!next_atom) - 1


let new_atom ((_,_,_,mh) as m) =
  let a = (!next_atom) in
  incr next_atom;
  Hashtbl.add atom_hash mh a;
  Hashtbl.add atom_hash_rev a m;
  if (!verbosity > 4) then
    if (!verbosity > 8) then Printf.printf "Atom %d: %s\n" a (ftrma_str m)
    else (Printf.printf "Atom %d\n" a);
  List.iter (fun f -> f m a) !new_atom_actions;
  a

let get_atom ((_,_,_,mh) as m) =
  try Hashtbl.find atom_hash mh
  with Not_found -> new_atom m

let assignedp ((_,_,_,mh) as m) =
  match fneg_body m with
    Some (_,_,_,nh) -> Hashtbl.mem atom_hash nh
  | None -> Hashtbl.mem atom_hash mh

let atom_to_trm = Hashtbl.find atom_hash_rev

let get_literal m =
  match fneg_body m with
    Some n -> - (get_atom n)
  | None -> get_atom m

let literal_to_trm l =
  if (l < 0)
  then fneg (Hashtbl.find atom_hash_rev (- l))
  else Hashtbl.find atom_hash_rev l


let print_minisat eval b i m =
  let pol v = if v = 0 then "+" else "-"
  and v = eval i in
  match b with
    (*** 0 means true ***)
    true  -> if (v = 0) then Printf.printf "%d %s\n" i (ftrma_str m)
  | false -> if (v < 2) then Printf.printf "%d%s %s\n" i (pol v) (ftrma_str m)

let print_model eval b = Hashtbl.iter (print_minisat eval b) atom_hash_rev

